/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 library
* Version		: v1.2
* Last updated	: 11.06.23
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "globals.h"
#include "lib_dmx.h"
#include <avr/wdt.h>

// ********************* local definitions *********************

enum {IDLE, BREAK, STARTB, STARTADR, ERR};			//DMX states

		 uint8_t 	 gDmxState;
		 uint16_t	 DmxRxCount;
		 uint8_t	 DmxTxCount;
		 uint16_t	 TimeBuf;
		 



// *************** DMX Initialisation ****************
void init_DMX_RX(void)
{
DDRD  |= (1<<2);
PORTD &= ~(1<<2);								//enable reception
UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<RXEN)|(1<<RXCIE);
DmxAddress= 1;
gDmxState= IDLE;
DmxRxCount= 0;
gFlags &= ~(1<<SIGNAL);							//init with "no signal present"

uint8_t i;
for (i=0; i<sizeof(DmxField); i++)
	{
	DmxField[i]= 0;
	}
}


void init_DMX_TX(void)
{
//Ports
DDRD  |= (1<<2)|(1<<1);
PORTD |= (1<<2)|(1<<1);						//enable transmission

//USART
UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);					//250kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = 0;
//Data
DmxTxCount= 0xFF;

uint8_t i;
for (i=0; i<sizeof(DmxField); i++)
   {
	DmxField[i]= 0;
   }
}


void new_DMX_Tx(void)							//start transmission
{
if (PORTD &(1<<PD2))
	{											//only if transmitter enabled!
	UBRRH  = 0;
	UBRRL  = 21;								//45.5kbaud, 8N2
	UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
	UCSRB  = (1<<TXEN);
	UDR    = 0;									//send break
	UCSRB |=(1<<TXCIE);							//enable TX complete IRQ
	DmxTxCount  = 0;
	DmxField[0]= 0;								//make sure start code is 0
	}
}


void get_DmxLen(void)							//get break and MAB length
{
BreakLen= 0;
MabLen= 0;
if (gFlags &(1<<SIGNAL))						//make sure DMX is present
   {
	UCSRB  = 0;
	cli();
	wdt_reset();
	loop_until_bit_is_set(PIND, PD0);
	do {
		loop_until_bit_is_clear(PIND, PD0);		//falling edge
		TCNT1= 0;
		loop_until_bit_is_set(PIND, PD0);		//rising edge -> end of break
		}
	while (TCNT1 < 80);
	BreakLen= TCNT1 -1;
	loop_until_bit_is_clear(PIND, PD0);			//falling edge -> end of MAB
	MabLen= TCNT1 -BreakLen;
	sei();
	UCSRB  = (1<<RXEN)|(1<<RXCIE);
	gDmxState= IDLE;
	DmxRxCount= 0;
   }
}


// *************** DMX Reception ISR ****************
ISR (USART_RX_vect)
{
static  uint16_t RxCnt;							//internal DMX counter
        uint8_t  USARTstate= UCSRA;				//get state before data!
        uint8_t  DmxByte   = UDR;				//get data
        uint8_t  DmxState  = gDmxState;			//just load once from SRAM to increase speed
	
if (USARTstate &(1<<FE))						//check for break
	{
	UCSRA  &= ~(1<<FE);							//reset flag (necessary for simulation in AVR Studio)
	RxCnt   =  DmxAddress;						//reset channel counter (count channels before start address)
	gDmxState= BREAK;
	return;
	}

if (DmxState != ERR) DmxRxCount++;				//just count DMX bytes

if (DmxState == BREAK)
	{
	if (DmxByte == 0)
	   {
		gFlags |= (1<<SIGNAL);					//mark: we have DMX!
		DmxRefresh++;							//frame counter for DMX rate
		DmxSize= DmxRxCount -1;					//store size of universe
		DmxRxCount=  0;							//reset frame counter
		gDmxState= STARTB;						//normal start code detected
	   }
	else gDmxState= ERR;
	}
	
else if (DmxState == STARTB)
	{
	if (--RxCnt == 0)							//start address reached?
		{
		RxCnt= 1;								//set up counter for required channels
		DmxField[0]= DmxByte;					//get 1st DMX channel of device
		gDmxState= STARTADR;
		}
	}

else if (DmxState == STARTADR)
	{
	DmxField[RxCnt++]= DmxByte;					//get channel
	if (RxCnt >= sizeof(DmxField)) 				//all ch received?
		{
		gDmxState= IDLE;						//wait for next break
		}
	}
}



// ****************** DMX Transmission ISR ********************
ISR (USART_TX_vect)
{
_delay_us(5);
uint8_t TxCh= DmxTxCount;
//transmit RDM response
if (TxCh == 0)
   {
	UBRRL= ((F_OSC/4000)-1);				//250kbaud
   }
else if (TxCh >= DMX_TX_CHANNELS)
   {
	UCSRB=0;								//shut down transmitter
	return;
   }

UDR= DmxField[TxCh++];						//send data
DmxTxCount= TxCh;
}